@@ -7,23 +7,26 @@ from typing import Any, Callable, Hashable, IO, TypeVar, Union, overload, TYPE_C
 
 if TYPE_CHECKING:
     from typing import Coroutine, Literal, Sequence
+
     from linkmerce.common.tasks import TaskOption, TaskOptions
     from linkmerce.common.tasks import RequestLoop, RequestEach, RequestEachLoop
     from linkmerce.common.tasks import PaginateAll, RequestEachPages
+    from linkmerce.common.tasks import CursorAll, RequestEachCursor
 
     from requests import Session, Response
+    from requests.cookies import RequestsCookieJar
     from aiohttp.client import ClientSession, ClientResponse
+    from aiohttp.typedefs import LooseCookies
     from bs4 import BeautifulSoup
     import datetime as dt
 
 _KT = TypeVar("_KT", bound=Hashable)
 _VT = TypeVar("_VT", bound=Any)
-
 Headers = dict[_KT,_VT]
 Variables = dict[_KT,_VT]
+
 JsonObject = Union[dict, list]
 JsonSerialize = Union[dict, list, bytes, IO]
-ResponseComponent = Union[int, bytes, str, JsonObject, dict[str,str]]
 
 
 class Client:
@@ -35,8 +38,8 @@ class Client:
 ###################################################################
 
 class BaseSessionClient(Client, metaclass=ABCMeta):
-    method: str
-    url: str
+    method: str | None = None
+    url: str | None = None
 
     def __init__(
             self,
@@ -54,8 +57,8 @@ class BaseSessionClient(Client, metaclass=ABCMeta):
 
     def build_request_message(self, **kwargs) -> dict:
         return dict(filter(lambda x: x[1] is not None, [
-                ("method", self.method),
-                ("url", self.url),
+                ("method", self.method or kwargs["method"]),
+                ("url", self.url or kwargs["url"]),
                 ("params", self.build_request_params(**kwargs)),
                 ("data", self.build_request_data(**kwargs)),
                 ("json", self.build_request_json(**kwargs)),
@@ -119,7 +122,7 @@ class BaseSessionClient(Client, metaclass=ABCMeta):
             client: str = str(),
             mobile: bool = False,
             platform: str = str(),
-            metadata: Literal["cors", "navigate"] | dict[str,str] = "navigate",
+            metadata: Literal["cors", "navigate"] | dict[str,str] = "cors",
             https: bool = False,
             user_agent: str = str(),
             ajax: bool = False,
@@ -144,17 +147,21 @@ class BaseSessionClient(Client, metaclass=ABCMeta):
 
 
 class RequestSessionClient(BaseSessionClient):
-    method: str
-    url: str
+    method: str | None = None
+    url: str | None = None
 
     def request(
             self,
-            parse: Literal["status","content","text","json","headers","html"] | None = None,
+            method: str,
+            url: str,
+            params: dict | list[tuple] | bytes | None = None,
+            data: dict | list[tuple] | bytes | IO | None = None,
+            json: JsonSerialize | None = None,
+            headers: dict[str,str] = None,
+            cookies: dict | RequestsCookieJar = None,
             **kwargs
-        ) -> Response | ResponseComponent:
-        if parse is not None:
-            return getattr(self, f"request_{parse}")(**kwargs)
-        message = self.build_request_message(**kwargs)
+        ) -> Response:
+        message = dict(method=method, url=url, params=params, data=data, json=json, headers=headers, cookies=cookies, **kwargs)
         return self.get_session().request(**message)
 
     def request_status(self, **kwargs) -> int:
@@ -177,6 +184,15 @@ class RequestSessionClient(BaseSessionClient):
         with self.get_session().request(**message) as response:
             return response.json()
 
+    def request_json_safe(self, **kwargs) -> JsonObject:
+        message = self.build_request_message(**kwargs)
+        with self.get_session().request(**message) as response:
+            from json import JSONDecodeError
+            try:
+                return response.json()
+            except JSONDecodeError:
+                return None
+
     def request_headers(self, **kwargs) -> dict[str,str]:
         message = self.build_request_message(**kwargs)
         with self.get_session().request(**message) as response:
@@ -187,6 +203,11 @@ class RequestSessionClient(BaseSessionClient):
         from bs4 import BeautifulSoup
         return BeautifulSoup(response, features)
 
+    def request_excel(self, sheet_name: str | None = None, header: int = 1, warnings: bool = False, **kwargs) -> JsonObject:
+        response = self.request_content(**kwargs)
+        from linkmerce.utils.excel import excel2json
+        return excel2json(response, sheet_name, header, warnings)
+
     def with_session(func):
         @functools.wraps(func)
         def wrapper(self: RequestSessionClient, *args, **kwargs):
@@ -207,20 +228,24 @@ class RequestSessionClient(BaseSessionClient):
 
 
 class AiohttpSessionClient(BaseSessionClient):
-    method: str
-    url: str
+    method: str | None = None
+    url: str | None = None
 
     def request(self, *args, **kwargs):
         raise NotImplementedError("This feature does not support synchronous requests. Please use the request_async method instead.")
 
     async def request_async(
             self,
-            parse: Literal["status","content","text","json","headers","html"] | None = None,
+            method: str,
+            url: str,
+            params: dict | list[tuple] | bytes | None = None,
+            data: dict | list[tuple] | bytes | IO | None = None,
+            json: JsonSerialize | None = None,
+            headers: dict[str,str] = None,
+            cookies: dict | LooseCookies = None,
             **kwargs
-        ) -> ClientResponse | ResponseComponent:
-        if parse is not None:
-            return await getattr(self, f"request_async_{parse}")(**kwargs)
-        message = self.build_request_message(**kwargs)
+        ) -> ClientResponse:
+        message = dict(method=method, url=url, params=params, data=data, json=json, headers=headers, cookies=cookies, **kwargs)
         return await self.get_session().request(**message)
 
     async def request_async_status(self, **kwargs) -> int:
@@ -243,6 +268,15 @@ class AiohttpSessionClient(BaseSessionClient):
         async with self.get_session().request(**message) as response:
             return await response.json()
 
+    async def request_async_json_safe(self, **kwargs) -> JsonObject:
+        message = self.build_request_message(**kwargs)
+        async with self.get_session().request(**message) as response:
+            from json import JSONDecodeError
+            try:
+                return await response.json()
+            except JSONDecodeError:
+                return None
+
     async def request_async_headers(self, **kwargs) -> dict[str,str]:
         message = self.build_request_message(**kwargs)
         async with self.get_session().request(**message) as response:
@@ -253,6 +287,11 @@ class AiohttpSessionClient(BaseSessionClient):
         from bs4 import BeautifulSoup
         return BeautifulSoup(response, features)
 
+    async def request_async_excel(self, sheet_name: str | None = None, header: int = 1, warnings: bool = False, **kwargs) -> JsonObject:
+        response = await self.request_async_content(**kwargs)
+        from linkmerce.utils.excel import excel2json
+        return excel2json(response, sheet_name, header, warnings)
+
     def async_with_session(func):
         @functools.wraps(func)
         async def wrapper(self: AiohttpSessionClient, *args, **kwargs):
@@ -273,8 +312,8 @@ class AiohttpSessionClient(BaseSessionClient):
 
 
 class SessionClient(RequestSessionClient, AiohttpSessionClient):
-    method: str
-    url: str
+    method: str | None = None
+    url: str | None = None
 
 
 ###################################################################
@@ -283,7 +322,7 @@ class SessionClient(RequestSessionClient, AiohttpSessionClient):
 
 class TaskClient(Client):
     def __init__(self, options: TaskOptions = dict(), parser: Callable | None = None):
-        self.set_options(options)
+        self.set_options(options or self.default_options)
         self.set_parser(parser)
 
     ########################### Task Options ##########################
@@ -298,6 +337,10 @@ class TaskClient(Client):
         options = {key: value for key, value in kwargs.items() if value is not None}
         return options or self.get_options(name)
 
+    @property
+    def default_options(self) -> TaskOptions:
+        return dict()
+
     ############################## Parser #############################
 
     def get_parser(self) -> Callable:
@@ -317,10 +360,11 @@ class TaskClient(Client):
             condition: Callable[...,bool],
             count: int | None = None,
             delay: Literal["incremental"] | float | int | Sequence[int,int] | None = None,
-            loop_error: type | None = None,
+            raise_errors: type | Sequence[type] | None = None,
+            ignored_errors: type | Sequence[type] | None = None,
         ) -> RequestLoop:
         from linkmerce.common.tasks import RequestLoop
-        options = self.build_options("RequestLoop", count=count, delay=delay, loop_error=loop_error)
+        options = self.build_options("RequestLoop", count=count, delay=delay, raise_errors=raise_errors, ignored_errors=ignored_errors)
         return RequestLoop(func, condition, parser=self.parse, **options)
 
     def request_each(
@@ -359,11 +403,10 @@ class TaskClient(Client):
             delay: float | int | tuple[int,int] | None = None,
             limit: int | None = None,
             tqdm_options: dict | None = None,
-            count_error: type | None = None,
         ) -> PaginateAll:
         from linkmerce.common.tasks import PaginateAll
         options = self.build_options("PaginateAll",
-            page_start=page_start, delay=delay, limit=limit, tqdm_options=tqdm_options, count_error=count_error)
+            page_start=page_start, delay=delay, limit=limit, tqdm_options=tqdm_options)
         return PaginateAll(func, counter, max_page_size, parser=self.parse, **options)
 
     def request_each_pages(
@@ -381,14 +424,39 @@ class TaskClient(Client):
             options["page_options"] = self.build_options("PaginateAll", **page_options)
         return RequestEachPages(func, context, parser=self.parse, **options)
 
+    def cursor_all(
+            self,
+            func: Callable,
+            get_next_cursor: Callable[...,Any],
+            next_cursor: Any | None = None,
+            delay: float | int | tuple[int,int] | None = None,
+        ) -> CursorAll:
+        from linkmerce.common.tasks import CursorAll
+        options = self.build_options("CursorAll", next_cursor=next_cursor, delay=delay)
+        return CursorAll(func, get_next_cursor, parser=self.parse, **options)
+
+    def request_each_cursor(
+            self,
+            func: Callable | Coroutine,
+            context: Sequence[tuple[_VT,...] | dict[_KT,_VT]] | dict[_KT,_VT] = list(),
+            delay: float | int | tuple[int,int] | None = None,
+            tqdm_options: dict | None = None,
+            cursor_options: dict = dict(),
+        ) -> RequestEachCursor:
+        from linkmerce.common.tasks import RequestEachCursor
+        options = self.build_options("RequestEachCursor", delay=delay, tqdm_options=tqdm_options)
+        if "cursor_options" not in options:
+            options["cursor_options"] = self.build_options("CursorAll", **cursor_options)
+        return RequestEachCursor(func, context, parser=self.parse, **options)
+
 
 ###################################################################
 ############################ Extractor ############################
 ###################################################################
 
 class Extractor(SessionClient, TaskClient, metaclass=ABCMeta):
-    method: str
-    url: str
+    method: str | None = None
+    url: str | None = None
 
     def __init__(
             self,
@@ -426,14 +494,74 @@ class Extractor(SessionClient, TaskClient, metaclass=ABCMeta):
     def set_variables(self, variables: Variables = dict()):
         self.__variables = variables
 
+    def concat_path(self, url: str, *args: str) -> str:
+        for path in args:
+            if isinstance(path, str) and path:
+                url = (url[:-1] if url.endswith('/') else url) + '/' + (path[1:] if path.startswith('/') else path)
+        return url
+
+    def generate_date_range(
+            self,
+            start_date: dt.date | str,
+            end_date: dt.date | str | Literal[":start_date:"] = ":start_date:",
+            freq: Literal["D","W","M"] = "D",
+            format: str = "%Y-%m-%d",
+        ) -> list[dict[str,dt.date]] | dict[str,dt.date]:
+        from linkmerce.utils.date import date_range
+        ranges = date_range(start_date, (start_date if end_date == ":start_date:" else end_date), freq, format)
+        return ranges[0] if len(ranges) == 1 else ranges
+
     def generate_date_context(
             self,
             start_date: dt.date | str,
-            end_date: dt.date | str,
+            end_date: dt.date | str | Literal[":start_date:"] = ":start_date:",
             freq: Literal["D","W","M"] = "D",
             format: str = "%Y-%m-%d",
         ) -> list[dict[str,dt.date]] | dict[str,dt.date]:
         from linkmerce.utils.date import date_pairs
-        pairs = date_pairs(start_date, end_date, freq, format)
+        pairs = date_pairs(start_date, (start_date if end_date == ":start_date:" else end_date), freq, format)
         context = list(map(lambda values: dict(zip(["start_date","end_date"], values)), pairs))
         return context[0] if len(context) == 1 else context
+
+
+###################################################################
+########################### LoginHandler ##########################
+###################################################################
+
+class LoginHandler(Extractor):
+    cookies: dict = dict()
+
+    @abstractmethod
+    def login(self, **kwargs):
+        raise NotImplementedError("The 'login' method must be implemented.")
+
+    def extract(self, *args, **kwargs) -> Any:
+        raise NotImplementedError("Direct calls to extract method are not supported. Please use login method instead.")
+
+    def with_session(func):
+        @functools.wraps(func)
+        def wrapper(self: LoginHandler, *args, **kwargs):
+            if self.get_session() == "per_request":
+                try:
+                    return self._run_with_session(func, *args, **kwargs)
+                finally:
+                    cookies = self.get_cookies()
+                    self.set_session("per_request")
+                    self.set_cookies(cookies)
+            else:
+                return func(self, *args, **kwargs)
+        return wrapper
+
+    def set_cookies(self, cookies: str):
+        cookies = {kv.split('=')[0]: kv.split('=', 1)[1] for kv in cookies.split("; ")}
+        if self.get_session() == "per_request":
+            self.cookies.update(cookies)
+        else:
+            self.get_session().cookies.update(cookies)
+
+    def get_cookies(self) -> str:
+        if self.get_session() == "per_request":
+            cookies = self.cookies
+        else:
+            cookies = self.get_session().cookies
+        return "; ".join([f"{key}={value}" for key, value in cookies.items()])
