@@ -6,11 +6,10 @@ from typing import Any, Hashable, Iterable, Sequence, TypeVar, TYPE_CHECKING
 
 if TYPE_CHECKING:
     from typing import Callable, Coroutine, Literal
+    _SKIPPED = TypeVar("_SKIPPED", bound=None)
 
 _KT = TypeVar("_KT", bound=Hashable)
 _VT = TypeVar("_VT", bound=Any)
-_SKIPPED = TypeVar("_SKIPPED", bound=None)
-
 TaskOption = dict[_KT,_VT]
 TaskOptions = dict[_KT,TaskOption]
 
@@ -69,34 +68,46 @@ class RunLoop(Task):
             condition: Callable[...,bool],
             count: int | None = 1,
             delay: Literal["incremental"] | float | int | Sequence[int,int] = "incremental",
-            loop_error: type = RuntimeError,
+            raise_errors: type | Sequence[type] = tuple(),
+            ignored_errors: type | Sequence[type] = tuple(),
         ):
         self.func = func
         self.condition = condition
         self.count = count
         self.delay = delay
-        self.loop_error = loop_error
+        self.raise_errors = raise_errors
+        self.ignored_errors = ignored_errors
 
     def run(self, *args, **kwargs) -> Any:
         if not isinstance(self.count, int):
             return self._infinite_run(args, kwargs)
         for count in range(1, self.count+1):
-            result = self.func(*args, **kwargs)
-            if self.condition(result):
-                return result
-            else:
-                self._sleep(count)
+            try:
+                result = self.func(*args, **kwargs)
+                if self.condition(result):
+                    return result
+            except self.raise_errors as error:
+                raise error
+            except self.ignored_errors:
+                if count == self.count:
+                    return
+            self._sleep(count)
         self._raise_loop_error()
 
     async def run_async(self, *args, **kwargs) -> Any:
         if not isinstance(self.count, int):
-            raise self.loop_error("Invalid loop count provided.")
+            raise RuntimeError("Invalid loop count provided.")
         for count in range(1, self.count+1):
-            result = await self.func(*args, **kwargs)
-            if self.condition(result):
-                return result
-            else:
-                await self._sleep_async(count)
+            try:
+                result = await self.func(*args, **kwargs)
+                if self.condition(result):
+                    return result
+            except self.raise_errors as error:
+                raise error
+            except self.ignored_errors:
+                if count == self.count:
+                    return
+            await self._sleep_async(count)
         self._raise_loop_error()
 
     def _infinite_run(self, args: tuple = tuple(), kwargs: dict = dict()) -> Any:
@@ -114,7 +125,7 @@ class RunLoop(Task):
         if self.delay == "incremental":
             time.sleep(count)
         else:
-            from linkmerce.utils.tqdm import _get_seconds
+            from linkmerce.utils.progress import _get_seconds
             time.sleep(_get_seconds(self.delay))
 
     async def _sleep_async(self, count: int):
@@ -122,11 +133,11 @@ class RunLoop(Task):
         if self.delay == "incremental":
             await asyncio.sleep(count)
         else:
-            from linkmerce.utils.tqdm import _get_seconds
+            from linkmerce.utils.progress import _get_seconds
             await asyncio.sleep(_get_seconds(self.delay))
 
     def _raise_loop_error(self):
-        raise self.loop_error("Exceeded maximum retry attempts without success.")
+        raise RuntimeError("Exceeded maximum retry attempts without success.")
 
 
 class RequestLoop(RunLoop, Request):
@@ -137,32 +148,21 @@ class RequestLoop(RunLoop, Request):
             parser: Callable | None = None,
             count: int | None = 1,
             delay: Literal["incremental"] | float | int | Sequence[int,int] = "incremental",
-            loop_error: type = RuntimeError,
+            raise_errors: type | Sequence[type] = tuple(),
+            ignored_errors: type | Sequence[type] = tuple(),
         ):
-        RunLoop.__init__(self, func, condition, count, delay, loop_error)
+        RunLoop.__init__(self, func, condition, count, delay, raise_errors, ignored_errors)
         self.parser = parser
 
     def run(self, *args, **kwargs) -> Any:
-        if not isinstance(self.count, int):
-            return self._infinite_run(args, kwargs)
-        for count in range(1, self.count+1):
-            result = self.func(*args, **kwargs)
-            if self.condition(result):
-                return self._parse(result, args, kwargs)
-            else:
-                self._sleep(count)
-        self._raise_loop_error()
+        result = super().run(*args, **kwargs)
+        if result is not None:
+            return self._parse(result, args, kwargs)
 
     async def run_async(self, *args, **kwargs) -> Any:
-        if not isinstance(self.count, int):
-            raise self.loop_error("Invalid loop count provided.")
-        for count in range(1, self.count+1):
-            result = await self.func(*args, **kwargs)
-            if self.condition(result):
-                return self._parse(result, args, kwargs)
-            else:
-                await self._sleep_async(count)
-        self._raise_loop_error()
+        result = await super().run_async(*args, **kwargs)
+        if result is not None:
+            return self._parse(result, args, kwargs)
 
     def parse(self, parser: Callable | None = None) -> RequestLoop:
         return self.setattr("parser", parser)
@@ -190,17 +190,17 @@ class ForEach(Task):
         self.concat_how = "auto"
 
     def run(self) -> list:
-        from linkmerce.utils.tqdm import gather
+        from linkmerce.utils.progress import gather
         results = gather(self.func, self.array, self.kwargs, self.delay, self.tqdm_options)
         return self._concat_results(results)
 
     async def run_async(self) -> list:
-        from linkmerce.utils.tqdm import gather_async
+        from linkmerce.utils.progress import gather_async
         results = await gather_async(self.func, self.array, self.kwargs, self.delay, self.limit, self.tqdm_options)
         return self._concat_results(results)
 
     def expand(self, **map_kwargs: Iterable[_VT]) -> ForEach:
-        from linkmerce.utils.tqdm import _expand_kwargs
+        from linkmerce.utils.progress import _expand_kwargs
         array = _expand_kwargs(**map_kwargs)
         return self.setattr("array", array)
 
@@ -211,9 +211,14 @@ class ForEach(Task):
         return self.setattr("concat_how", how)
 
     def _concat_results(self, results: list) -> list:
-        if (self.concat_how == "always") or ((self.concat_how == "auto") and all(map(lambda x: isinstance(x, Sequence), results))):
+        def chain_list(iterable: list):
             from itertools import chain
-            return list(chain.from_iterable(results))
+            return list(chain.from_iterable(iterable))
+        if self.concat_how == "always":
+            return chain_list(results)
+        elif self.concat_how == "auto":
+            iterable = [result for result in results if isinstance(result, Sequence)]
+            return chain_list(iterable) if iterable else results
         else:
             return results
 
@@ -247,7 +252,7 @@ class RequestEach(ForEach, Request):
 
     def run(self) -> list | Any:
         if isinstance(self.context, Sequence):
-            from linkmerce.utils.tqdm import gather
+            from linkmerce.utils.progress import gather
             results = gather(self.callable, self.context, self.kwargs, self.delay, self.tqdm_options)
             return self._concat_results(results)
         elif isinstance(self.context, dict):
@@ -257,7 +262,7 @@ class RequestEach(ForEach, Request):
 
     async def run_async(self) -> list | Any:
         if isinstance(self.context, Sequence):
-            from linkmerce.utils.tqdm import gather_async
+            from linkmerce.utils.progress import gather_async
             results = await gather_async(self.coroutine, self.context, self.kwargs, self.delay, self.limit, self.tqdm_options)
             return self._concat_results(results)
         elif isinstance(self.context, dict):
@@ -289,7 +294,7 @@ class RequestEach(ForEach, Request):
         return sequential, non_sequential
 
     def _expand_context(self, mapping: dict[_KT,Sequence]) -> Sequence[dict[_KT,_VT]]:
-        from linkmerce.utils.tqdm import _expand_kwargs
+        from linkmerce.utils.progress import _expand_kwargs
         context = self._get_sequential_context()
         if context:
             context = _expand_kwargs(context_=context, **mapping)
@@ -370,7 +375,6 @@ class PaginateAll(ForEach, Request):
             delay: float | int | tuple[int,int] = 0.,
             limit: int | None = None,
             tqdm_options: dict = dict(),
-            count_error: type = ValueError,
         ):
         self.func = func
         self.counter = counter
@@ -380,14 +384,13 @@ class PaginateAll(ForEach, Request):
         self.delay = delay
         self.limit = limit
         self.tqdm_options = tqdm_options
-        self.count_error = count_error
         self.concat_how = "never"
 
     def run(self, page: _SKIPPED = None, page_size: _SKIPPED = None, **kwargs) -> list:
         kwargs["page_size"] = self.max_page_size
         results, total_count = self._run_with_count(page=self.page_start, **kwargs)
         if isinstance(total_count, int) and (total_count > self.max_page_size):
-            from linkmerce.utils.tqdm import gather
+            from linkmerce.utils.progress import gather
             func = self._run_without_count
             pages = map(lambda page: dict(page=page), self._generate_next_pages(total_count))
             results = [results] + gather(func, pages, kwargs, self.delay, self.tqdm_options)
@@ -399,7 +402,7 @@ class PaginateAll(ForEach, Request):
         kwargs["page_size"] = self.max_page_size
         results, total_count = await self._run_async_with_count(page=self.page_start, **kwargs)
         if isinstance(total_count, int) and (total_count > self.max_page_size):
-            from linkmerce.utils.tqdm import gather_async
+            from linkmerce.utils.progress import gather_async
             func = self._run_async_without_count
             pages = map(lambda page: dict(page=page), self._generate_next_pages(total_count))
             results = [results] + (await gather_async(func, pages, kwargs, self.delay, self.limit, self.tqdm_options))
@@ -415,23 +418,23 @@ class PaginateAll(ForEach, Request):
 
     def _generate_next_pages(self, total_count: int) -> Iterable[int]:
         from math import ceil
-        return range(self.page_start + 1, ceil(total_count / self.max_page_size))
+        return range(self.page_start + 1, ceil(total_count / self.max_page_size) + 1)
 
-    def _run_with_count(self, *args, **kwargs) -> tuple[Any,int]:
-        results = self.func(*args, **kwargs)
-        return self._parse(results, args, kwargs), self.counter(results)
+    def _run_with_count(self, **kwargs) -> tuple[Any,int]:
+        results = self.func(**kwargs)
+        return self._parse(results, kwargs=kwargs), self.counter(results, **kwargs)
 
-    def _run_without_count(self, *args, **kwargs) -> Any:
-        results = self.func(*args, **kwargs)
-        return self._parse(results, args, kwargs)
+    def _run_without_count(self, **kwargs) -> Any:
+        results = self.func(**kwargs)
+        return self._parse(results, kwargs=kwargs)
 
-    async def _run_async_with_count(self, *args, **kwargs) -> tuple[Any,int]:
-        results = await self.func(*args, **kwargs)
-        return self._parse(results, args, kwargs), self.counter(results)
+    async def _run_async_with_count(self, **kwargs) -> tuple[Any,int]:
+        results = await self.func(**kwargs)
+        return self._parse(results, kwargs=kwargs), self.counter(results, **kwargs)
 
-    async def _run_async_without_count(self, *args, **kwargs) -> Any:
-        results = await self.func(*args, **kwargs)
-        return self._parse(results, args, kwargs)
+    async def _run_async_without_count(self, **kwargs) -> Any:
+        results = await self.func(**kwargs)
+        return self._parse(results, kwargs=kwargs)
 
 
 class RequestEachPages(RequestEach):
@@ -483,3 +486,88 @@ class RequestEachPages(RequestEach):
             return self.setattr("func", self.func.concat(how), "concat_how", how)
         else:
             return self.setattr("concat_how", how)
+
+
+###################################################################
+############################## Cursor #############################
+###################################################################
+
+class CursorAll(RunLoop, ForEach, Request):
+    def __init__(
+            self,
+            func: Callable,
+            get_next_cursor: Callable[...,Any],
+            next_cursor: Any | None = None,
+            parser: Callable | None = None,
+            delay: float | int | tuple[int,int] = 0.,
+        ):
+        self.func = func
+        self.get_next_cursor = get_next_cursor
+        self.next_cursor = next_cursor
+        self.parser = parser
+        self.delay = delay
+        self.concat_how = "never"
+
+    def run(self, next_cursor: _SKIPPED = None, **kwargs) -> list:
+        results, next_cursor = list(), self.next_cursor
+        while (next_cursor is not None) or (not results):
+            result, next_cursor = self._run_with_cursor(next_cursor=next_cursor, **kwargs)
+            results.append(result)
+            self._sleep(self.delay)
+        return self._concat_results(results)
+
+    async def run_async(self):
+        raise NotImplementedError("This task does not support asynchronous execution. Please use the run method instead.")
+
+    def parse(self, parser: Callable | None = None) -> CursorAll:
+        return self.setattr("parser", parser)
+
+    def concat(self, how: Literal["always","never","auto"] = "auto") -> CursorAll:
+        return self.setattr("concat_how", how)
+
+    def _run_with_cursor(self, **kwargs) -> tuple[Any,Any]:
+        results = self.func(**kwargs)
+        return self._parse(results, kwargs=kwargs), self.get_next_cursor(results, **kwargs)
+
+
+class RequestEachCursor(RequestEach):
+    def __init__(
+            self,
+            func: Callable | Coroutine,
+            context: Sequence[tuple[_VT,...] | dict[_KT,_VT]] | dict[_KT,_VT] = list(),
+            parser: Callable | None = None,
+            delay: float | int | tuple[int,int] = 0.,
+            tqdm_options: dict = dict(),
+            cursor_options: dict = dict(tqdm_options=dict(disable=True)),
+        ):
+        super().__init__(func, context, parser, delay, None, tqdm_options)
+        self.cursor_options = cursor_options
+
+    @property
+    def callable(self) -> Callable:
+        if isinstance(self.func, CursorAll):
+            return self.func.run
+        else:
+            return Request(self.func, self.parser).run
+
+    async def run_async(self):
+        raise NotImplementedError("This task does not support asynchronous execution. Please use the run method instead.")
+
+    def parse(self, parser: Callable | None = None) -> RequestEachCursor:
+        return self.setattr("parser", parser)
+
+    def expand(self, **map_kwargs: _VT) -> RequestEachCursor:
+        return super().expand(**map_kwargs)
+
+    def partial(self, **kwargs: _VT) -> RequestEachCursor:
+        return self.setattr("kwargs", kwargs)
+
+    def all_cursor(self, get_next_cursor: Callable[...,Any], next_cursor: Any | None = None, **kwargs) -> RequestEachCursor:
+        cursor_all = CursorAll(self.func, get_next_cursor, next_cursor, self.parser, **(kwargs or self.cursor_options))
+        return self.setattr("func", cursor_all)
+
+    def concat(self, how: Literal["always","never","auto"] = "auto") -> RequestEachCursor:
+        if isinstance(self.func, PaginateAll):
+            return self.setattr("func", self.func.concat(how), "concat_how", how)
+        else:
+            return self.setattr("concat_how", how)
